//Windows header
#ifdef WIN32
#include <windows.h>
#endif

// GLEW header for OpenGL extentions loading
#include "GL/glew.h"
// OpenGL header
#include <GL/gl.h>
// Glut for display window managment
#include <gl/glut.h>

#include <string>
#include <iostream>
#include <fstream>

///////////////////////////////////////////
std::string loadTextFile(const char *name);
GLuint createShader(const char *fileName, GLuint shaderType, GLuint shaderID=0);
void checkProgramInfos(GLuint programID, GLuint stat);
GLuint createShaderProgram(const char *fileNameVS, const char *fileNameFS, GLuint programID=0);
///////////////////////////////////////////

//Size of the string, the shorter is better
#define STRING_BUFFER_SIZE 2048
char stringBuffer[STRING_BUFFER_SIZE];
GLuint vertexShaderID;
GLuint fragmentShaderID;


//GLSL shader program creation
GLuint createShaderProgram(const char *fileNameVS, const char *fileNameFS, GLuint programID){
	bool reload=programID!=0;
	if(!reload){
		// Create GLSL program
		programID=glCreateProgram();
	}

	// Create vertex shader
	if(!reload){
		vertexShaderID=createShader(fileNameVS, GL_VERTEX_SHADER, 0);
	}else{
		createShader(fileNameVS, GL_VERTEX_SHADER, vertexShaderID);
	}

	// Attach vertex shader to program object
	glAttachShader(programID, vertexShaderID);

	// Create fragment shader
	if(!reload)
		fragmentShaderID=createShader(fileNameFS, GL_FRAGMENT_SHADER, 0);
	else
		createShader(fileNameFS, GL_FRAGMENT_SHADER, fragmentShaderID);

	// Attach fragment shader to program object
	glAttachShader(programID, fragmentShaderID);

	// Link all shaders togethers into the GLSL program
	glLinkProgram(programID);
	checkProgramInfos(programID, GL_LINK_STATUS);

	// Validate program executability giving current OpenGL states
  glValidateProgram(programID);
	checkProgramInfos(programID, GL_VALIDATE_STATUS);

	return programID;
}


//GLSL shader creation (of a certain type, vertex shader, fragment shader oe geometry shader)
GLuint createShader(const char *fileName, GLuint shaderType, GLuint shaderID){
	if(shaderID==0){
		shaderID=glCreateShader(shaderType);
	}
	
	std::string shaderSource=loadTextFile(fileName);

	//Passing shader source code to GL
	//Source used for "shaderID" shader, there is only "1" source code and the string is NULL terminated (no sizes passed)
	const char *src=shaderSource.c_str();
	glShaderSource(shaderID, 1, &src, NULL);

	//Compile shader object
	glCompileShader(shaderID);

	//Check compilation status
	GLint ok;
	glGetShaderiv(shaderID, GL_COMPILE_STATUS, &ok);
	if (!ok){
		int ilength;
		glGetShaderInfoLog(shaderID, STRING_BUFFER_SIZE, &ilength, stringBuffer);
		std::cout<<"Compilation error ("<<fileName<<") : "<<stringBuffer; 
	}

	return shaderID;
}


//Text file loading for shaders sources
std::string loadTextFile(const char *name){
	//Source file reading
	std::string buff("");
	std::ifstream file;
	file.open(name);

	if(file.fail())
		std::cout<<"loadFile: unable to open file: "<<name;
	
	buff.reserve(1024*1024);

	std::string line;
	while(std::getline(file, line)){
		buff += line + "\n";
	}

	const char *txt=buff.c_str();

	return std::string(txt);
}

void checkProgramInfos(GLuint programID, GLuint stat){
	GLint ok = 0;
	glGetProgramiv(programID, stat, &ok);
	if (!ok){
		int ilength;
		glGetProgramInfoLog(programID, STRING_BUFFER_SIZE, &ilength, stringBuffer);
		std::cout<<"Program error :\n"<<stringBuffer<<"\n"; 
	}
}

